import matplotlib.patches
import pandas as pd
from tkinter import ttk

from matplotlib import pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from tkinter import messagebox

import data
import tkinter as tk


class Spoticult(tk.Tk):
    def __init__(self):
        super().__init__()
        self.configure(background='#f5f5f5')
        reader = data.DataReader('cleanned_song.csv')
        self.df = reader.read()
        self.title("SpotiCult")
        self.geometry("1800x1000")
        self.grid_columnconfigure(1, weight=1)
        #treeview for data
        self.trv = ttk.Treeview(self, selectmode='browse', name='my_treeview', columns=list(self.df.columns),
                                displaycolumns=list(self.df.columns), height=10)
        self.trv.grid(row=2, column=1, columnspan=2, padx=20, pady=20, sticky='nsew')
        self.trv['show'] = 'headings'
        self.trv.bind("<ButtonRelease-1>", self.show_graph)

        for col in list(self.df.columns):
            self.trv.column(col, width=90, anchor='c')
            self.trv.heading(col, text=col)

        for j, row in self.df.iterrows():
            values = list(row)
            self.trv.insert("", 'end', iid=j, values=values)

        #treeview for playlist
        self.playlist_trv = ttk.Treeview(self, selectmode='browse', name='my_playlist',
                                         columns=list(self.df.columns),
                                         displaycolumns=list(self.df.columns), height=10)
        self.playlist_trv.grid(row=3, column=1, columnspan=2, padx=20, pady=20, sticky='nsew')
        self.playlist_trv['show'] = 'headings'


        for col in list(self.df.columns):
            self.playlist_trv.column(col, width=90, anchor='c')
            self.playlist_trv.heading(col, text=col)



        # Create figure canvas for graph 1
        self.fig = Figure(figsize=(5, 4), dpi=100)
        self.ax = self.fig.add_subplot(111)
        self.canvas = FigureCanvasTkAgg(self.fig, master=self)
        self.canvas.get_tk_widget().grid(row=2, column=3, padx=20, pady=20, sticky='nsew')
        self.canvas.draw()

        # Create figure canvas for graph 2

        self.fig2 = Figure(figsize=(5, 4), dpi=100)
        self.ax2 = self.fig2.add_subplot(111)
        self.canvas2 = FigureCanvasTkAgg(self.fig2, master=self)
        self.canvas2.get_tk_widget().grid(row=3, column=3, padx=20, pady=20, sticky='nsew')
        self.canvas2.draw()


        # Create widgets

        self.entry = tk.Entry(self, font=('Arial', 14))
        self.entry.grid(row=1, column=1, pady=20, padx=30, sticky='w')
        self.search_button = tk.Button(self, text='Search By Track Name', command=self.search, bg='#0074D9', fg='white', font=('Arial', 14))
        self.search_button.grid(row=1, column=3, pady=20, sticky='w')
        self.add_button = tk.Button(self, text='Add to Playlist', command=self.add_to_playlist, bg='#0074D9',
                                    fg='white', font=('Arial', 14))
        self.add_button.grid(row=1, column=3, pady=20, padx=260, sticky='w')
        self.show_button = tk.Button(self, text="Show Playlist's Graph", command=self.show_playlist_graph, bg='#0074D9', fg='white', font=('Arial', 14))
        self.show_button.grid(row=1, column=3, pady=20, padx=440, sticky='w')
        self.histogram_button = tk.Button(self, text='Show histogram', command=self.show_histogram, bg='#0074D9', fg='white', font=('Arial', 14))
        self.histogram_button.grid(row=1, column=3, pady=20, padx=660, sticky='w')

        # Center window on screen
        self.eval('tk::PlaceWindow %s center' % self.winfo_toplevel())

    def search(self):
        search_term = self.entry.get()
        if search_term:
            # get all the values in the track name column
            track_names = self.df['Track Name'].values
            # find the indices of the matching track names
            matching_indices = [i for i, name in enumerate(track_names) if search_term.lower() in name.lower()]
            # clear the treeview
            self.trv.delete(*self.trv.get_children())
            # insert the matching rows into the treeview
            for i in matching_indices:
                row = self.df.iloc[i]
                values = list(row)
                self.trv.insert("", 'end', iid=i, values=values)
        else:
            # if the search term is empty, display all the rows
            self.trv.delete(*self.trv.get_children())
            for i, row in self.df.iterrows():
                values = list(row)
                self.trv.insert("", 'end', iid=i, values=values)

    def add_to_playlist(self):
        # Get the selected items in the treeview widget
        selected_items = self.trv.selection()
        if selected_items:
            # Add the selected rows to the playlist treeview
            for item in selected_items:
                row = self.trv.item(item, 'values')
                song_id = row[0]
                song_exists = False
                for child in self.playlist_trv.get_children():
                    if song_id == self.playlist_trv.item(child, 'values')[0]:
                        song_exists = True
                        break
                if not song_exists:
                    self.playlist_trv.insert("", 'end', values=row)
                else:
                    tk.messagebox.showwarning("Warning", f"The song {row[1]} is already in the playlist.")
        else:
            # If no items are selected, show an error message
            tk.messagebox.showerror("Error", "Please select at least one song.")

    def show_graph(self, event):
        # Get the selected item in the treeview widget
        selected_item = self.trv.focus()
        values = self.trv.item(selected_item, 'values')
        if values:
            # Clear the previous plot
            self.ax.clear()

            # Plot the selected row
            x_data = ['Popularity', 'danceability', 'energy', 'key', 'loudness', 'speechiness', 'acousticness',
                      'instrumentalness', 'liveness', 'valence',]
            y_data = [float(values[i]) for i in range(2,12)]
            colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'orange', 'purple', 'pink']
            self.ax.bar(x_data, y_data, color=colors, alpha=0.8)

            # Set the plot title and labels
            self.ax.set_title(f"Data for {values[1]}")
            self.ax.set_xlabel("Attribute")
            self.ax.set_ylabel("Percentage")
            self.ax.set_ylim([0, 100])
            self.ax.set_yticks([i * 10 for i in range(11)])

            # legend
            legend_handles = [matplotlib.patches.Patch(facecolor=colors[i], label=x_data[i]) for i in range(len(colors))]
            self.ax.legend(handles=legend_handles)

            # Redraw the canvas
            self.canvas.draw()

    def show_playlist_graph(self):
        self.ax2.clear()
        self.playlist_data = self.create_playlist_dataframe()
        self.mean_attribute = self.playlist_data.drop(['Artist Name', 'Track Name', 'Class', 'tempo'], axis=1)
        self.mean_attribute = self.mean_attribute.astype(float)
        means = []
        for column in self.mean_attribute.columns:
            means.append(self.mean_attribute[column].mean())
        x_data = ['Popularity', 'danceability', 'energy', 'key', 'loudness', 'speechiness', 'acousticness',
                  'instrumentalness', 'liveness', 'valence', ]
        y_data = means
        colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'orange', 'purple', 'pink']
        self.ax2.bar(x_data, y_data, color=colors, alpha=0.8)

        # Set the plot title and labels
        self.ax2.set_title("Playlist Data")
        self.ax2.set_xlabel("Attribute")
        self.ax2.set_ylabel("Percentage")
        self.ax2.set_ylim([0, 100])
        self.ax2.set_yticks([i * 10 for i in range(11)])
        # legend
        legend_handles = [matplotlib.patches.Patch(facecolor=colors[i], label=x_data[i]) for i in range(len(colors))]
        self.ax2.legend(handles=legend_handles)

        # Redraw the canvas
        self.canvas2.draw()

    def create_playlist_dataframe(self):
        #create new dataframe from the playlist
        cols = self.playlist_trv["columns"]
        data = {col: [] for col in cols}
        for row in self.playlist_trv.get_children():
            values = self.playlist_trv.item(row, "values")
            for i in range(len(cols)):
                data[cols[i]].append(values[i])
        playlist_df = pd.DataFrame(data)

        return playlist_df


    def show_histogram(self):
        # Plot the histogram
        plt.hist(self.df["Class"], bins=11)
        plt.title("Distribution of Class Attribute")
        plt.xlabel("Class")
        plt.ylabel("Frequency")

        # Create the legend
        handles = [plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7),
                   plt.Rectangle((0, 0), 1, 1, ec="k", alpha=0.7)]
        labels = ["0.Acoustic/Folk", "1.Alt", "2.Blues", "3.Bollywood_3", "4.Country", "5.HipHop",
                  "6.Indie Alt", "7.Instrumental", "8.Metal", "9.Pop", "10.Rock"]
        plt.legend(handles, labels, loc="best", fancybox=True, framealpha=0.7)

        plt.show()



